/*
 * qsasl.h - SASL wrapper
 * Copyright (C) 2003  Justin Karneges
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef CS_QSASL_H
#define CS_QSASL_H

#include"bytestream.h"

class QHostAddress;
class QStringList;

class QSASL : public ByteStream
{
	Q_OBJECT
public:
	enum Error { ErrSASL = ErrCustom };
	QSASL(QObject *parent=0);
	~QSASL();

	QString service() const;
	QString host() const;
	QString user() const;
	QString pass() const;
	QString realm() const;
	bool allowPlainText() const;
	void setServiceHost(const QString &service, const QString &host);
	void setUserPass(const QString &user, const QString &pass);
	void setRealm(const QString &realm);
	void setLocalIP(const QHostAddress &addr, Q_UINT16 port);
	void setRemoteIP(const QHostAddress &addr, Q_UINT16 port);
	void setAllowPlainText(bool);

	void reset();
	bool begin(const QStringList &methods);

	void putChallenge(const QByteArray &);

	ByteStream *byteStream() const;
	void setByteStream(ByteStream *);

	bool isSecure() const;
	bool isOpen() const;
	void write(const QByteArray &);

signals:
	void authenticated();
	void replyReady(const QByteArray &);

protected:
	int tryWrite();

private slots:
	void bs_readyRead();
	void bs_bytesWritten(int);

private:
	class Private;
	Private *d;

	void handle_step(int, const char *, unsigned int);
};

#endif
